// MBC.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include <winsock.h>
#include <\HAPTools\SDK\include\iobase.h>
#include <\HAPTools\SDK\include\MB_API.h>

// This enables scan time monitoring. This is useful for development, but you will 
// need to remove when deploying.
#define MONITOR_SCAN_TIME

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	char CharBuff[100];
	WCHAR WideCharBuff[100];

	OutputDebugString(TEXT("MailBox Sample Client started!"));

	// Set thread priority to highest. This dramatically improves scan consistency.
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);

	// Setup target address
	DWORD Addr = 0xFFFFFFFF;
	if(wcslen(lpCmdLine))
		{
		if(WideCharToMultiByte(CP_OEMCP, 0, lpCmdLine, -1, CharBuff, sizeof(CharBuff), NULL, NULL))
			Addr = inet_addr(CharBuff);
		}

	// Create and initialize the I/O base interface
	CIOBase IOB;
	IOB.Init();

	// Create mailbox and connection
	HMAILBOX hMB = CreateMailbox(0x7171);
	HMBC hMBC = CreateConnection(hMB, Addr, 100, 3);

/*
** TO DO: Do power on initialization here
*/


	// Turn on the run mode LED
	IOB.TurnOnRunLED();	

#ifdef MONITOR_SCAN_TIME
	int MinScan = 0x7FFFFFFF;
	int MaxScan = 0;
	float AvgScan = 0;
	int Count = 0;
	DWORD LastTime = GetTickCount();

	float AvgIdle = 0;
	int MinIdle = 0x7FFFFFFF;
	int MaxIdle = 0;
	int IdleCount = 0;
	DWORD LastIdle = 0;
#endif

	BYTE IOVal = 0, LastIOVal = 0;

	// Primary scan. Set Complete to TRUE to exit...
	BOOL Complete = FALSE;
	while(!Complete)
		{
		// Read base inputs
		IOB.ReadIO();

/*
** TO DO: Solve logic here
*/
		Complete = IOB.GetDI(0, 0); // I'm using slot 0, point 0 to terminate the scan

		IOVal = IOB.GetDI(0, 1) ? 0x02 : 0;
		IOVal |= IOB.GetDI(0, 2) ? 0x04 : 0;
		IOVal |= IOB.GetDI(0, 3) ? 0x08 : 0;
		IOVal |= IOB.GetDI(0, 4) ? 0x10 : 0;
		IOVal |= IOB.GetDI(0, 5) ? 0x20 : 0;
		IOVal |= IOB.GetDI(0, 6) ? 0x40 : 0;
		IOVal |= IOB.GetDI(0, 7) ? 0x80 : 0;

		// Send changes
		if(IOVal != LastIOVal)
			{
			wsprintf(WideCharBuff, L"Slot 0 = %d", IOVal);
			int len = wcslen(WideCharBuff)+1;
			WideCharToMultiByte(CP_OEMCP, 0, WideCharBuff, -1, CharBuff, sizeof(CharBuff), NULL, NULL);
			SendMsg(hMB, hMBC, (BYTE *)CharBuff, len);
			}

		LastIOVal = IOVal;

		// Print incoming
		while(IncomingMessagesAvailable(hMB))
			{
			WORD Length = sizeof(CharBuff);
			DWORD Source;
			if(GetIncomingMessage(hMB, (BYTE *)CharBuff, &Length, &Source))
				{
				MultiByteToWideChar(CP_OEMCP, 0, CharBuff, Length, WideCharBuff, sizeof(WideCharBuff));
				OutputDebugString(WideCharBuff);
				Sleep(5);
				}
			}


		// Write base outputs
		IOB.WriteIO();

		// Allow other stuff to execute. Since we have set
		// our priority to time critical, this must be here
		// to allow other things in the system to run. This
		// value can be pretty much anything.
		Sleep(10);

#ifdef MONITOR_SCAN_TIME
		DWORD CurTime = GetTickCount();
		int CurScan = CurTime - LastTime;
		MinScan = min(MinScan, CurScan);
		MaxScan = max(MaxScan, CurScan);
		if(Count + 1 == 0) Count = 0;
		AvgScan = ((AvgScan * Count) + CurScan) / (double)++Count;
		LastTime = CurTime;
#endif
		}

/*
** TO DO: Clean up here
*/


	// Turn off the run mode LED
	IOB.TurnOffRunLED();
	
	// Turn outputs off
	IOB.ClearOutputs();
	IOB.WriteIO();

	// Close mailbox stuff	
	DestroyConnection(hMB, hMBC);
	DestroyMailbox(hMB);

	// Clean up I/O base interface
	IOB.Exit();

#ifdef MONITOR_SCAN_TIME
	WCHAR Str[100];
	wsprintf(Str, TEXT("min scan = %d ms, average scan = %.2f ms, max scan = %d ms"), MinScan, AvgScan, MaxScan);
	OutputDebugString(Str);
	wsprintf(Str, TEXT("min idle = %d, average idle = %.2f, max idle = %d"), MinIdle, AvgIdle, MaxIdle);
	OutputDebugString(Str);
#endif

	OutputDebugString(TEXT("MailBox Sample Client exiting. Bye!"));
	return 0;
}

