// UserMain.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include <iobase.h>

// This enables scan time monitoring. This is useful for development, but you will 
// need to remove when deploying.
#define MONITOR_SCAN_TIME

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	OutputDebugString(TEXT("UserMain started!"));

	// Set thread priority to highest. This dramatically improves scan consistency.
	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);

	// Create and initialize the I/O base interface
	CIOBase IOB;
	IOB.Init();

/*
** TO DO: Do power on initialization here
*/


	// Turn on the run mode LED
	IOB.TurnOnRunLED();	

#ifdef MONITOR_SCAN_TIME
	int MinScan = 0x7FFFFFFF;
	int MaxScan = 0;
	float AvgScan = 0;
	int Count = 0;
	DWORD LastTime = GetTickCount();

	float AvgIdle = 0;
	int MinIdle = 0x7FFFFFFF;
	int MaxIdle = 0;
	int IdleCount = 0;
	DWORD LastIdle = 0;
#endif

	// Primary scan. Set Complete to TRUE to exit...
	BOOL Complete = FALSE;
	while(!Complete)
		{
		// Read base inputs
		IOB.ReadIO();

/*
** TO DO: Solve logic here
*/
		Complete = IOB.GetDI(0, 0); // I'm using slot 0, point 0 to terminate the scan

		// Copying points 1-7 in slot 0 to 1-7 in slot 1
		IOB.SetDO(1, 1, IOB.GetDI(0, 1));
		IOB.SetDO(1, 2, IOB.GetDI(0, 2));
		IOB.SetDO(1, 3, IOB.GetDI(0, 3));
		IOB.SetDO(1, 4, IOB.GetDI(0, 4));
		IOB.SetDO(1, 5, IOB.GetDI(0, 5));
		IOB.SetDO(1, 6, IOB.GetDI(0, 6));
		IOB.SetDO(1, 7, IOB.GetDI(0, 7));


		// Write base outputs
		IOB.WriteIO();

		// Allow other stuff to execute. Since we have set
		// our priority to time critical, this must be here
		// to allow other things in the system to run. This
		// value can be pretty much anything.
		Sleep(1);

#ifdef MONITOR_SCAN_TIME
		DWORD CurTime = GetTickCount();
		int CurScan = CurTime - LastTime;
		MinScan = min(MinScan, CurScan);
		MaxScan = max(MaxScan, CurScan);
		if(Count + 1 == 0) Count = 0;
		AvgScan = ((AvgScan * Count) + CurScan) / (double)++Count;
		LastTime = CurTime;

		int CurIdle = IOB.Idles - LastIdle;
		if(CurIdle)
			{
			MinIdle = min(MinIdle, CurIdle);
			MaxIdle = max(MaxIdle, CurIdle);
			if(IdleCount + 1 == 0) IdleCount = 0;
			AvgIdle = ((AvgIdle * IdleCount) + CurIdle) / (double)++IdleCount;
			LastIdle = IOB.Idles;
			}
#endif
		}

/*
** TO DO: Clean up here
*/


	// Turn off the run mode LED
	IOB.TurnOffRunLED();
	
	// Turn outputs off
	IOB.ClearOutputs();
	IOB.WriteIO();
	
	// Clean up I/O base interface
	IOB.Exit();

#ifdef MONITOR_SCAN_TIME
	WCHAR Str[100];
	wsprintf(Str, TEXT("min scan = %d ms, average scan = %.2f ms, max scan = %d ms"), MinScan, AvgScan, MaxScan);
	OutputDebugString(Str);
	wsprintf(Str, TEXT("min idle = %d, average idle = %.2f, max idle = %d"), MinIdle, AvgIdle, MaxIdle);
	OutputDebugString(Str);
#endif
	
	OutputDebugString(TEXT("UserMain exiting. Bye!"));
	return 0;
}

