#ifndef __COMMAPI_H
#define __COMMAPI_H

#ifdef WPLCCOMM_EXPORTS
#define WPLCCOMM_API __declspec(dllexport)
#else
#define WPLCCOMM_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void* HCOMMPORT;

/*
** CreateCommPort - Creates an uninitialized comm port.
**
** Note: Port must be opened and initialized with OpenCommPort before use.
*/
WPLCCOMM_API HCOMMPORT CreateCommPort();

/*
** DestroyCommPort - Destroys a previously opened comm port.
**
** hPort: Handle of port to destroy.
**
*/
WPLCCOMM_API void DestroyCommPort(HCOMMPORT hPort);

/*
** OpenCommPort - Opens and initializes a comm port.
**
** hPort: Handle of port to initialize.
** pPort: Pointer to wide string containing port to open. Example L"COM1:". Use L"COM0:" for COM10.
** BaudRate: Baud rate of port. Uses standard Win32 defines. Example CBR_9600. See CBR_XXX defines in Win32 docs.
** ByteSize: Size of data byte. Normally 8.
** Parity: Parity of port. Uses standard Win32 defines. Example NOPARITY.
** StopBits: Number of stop bits for port. Uses standard Win32 defines. Example ONESTOPBIT.
**
*/
WPLCCOMM_API BOOL OpenCommPort(HCOMMPORT hPort, WCHAR *pPort, DWORD BaudRate, BYTE ByteSize, BYTE Parity, BYTE StopBits);

/*
** CloseCommPort - Closes a port but doesn't destroy the object
**
** hPort: Handle of port to close.
**
*/
WPLCCOMM_API void CloseCommPort(HCOMMPORT hPort);

/*
** WriteComm - Writes a data string to the comm port.
**
** hPort: Handle of port to write to.
** pData: Data buffer to write.
** Count: Number of bytes to write.
**
*/
WPLCCOMM_API void WriteComm(HCOMMPORT hPort, void *pData, int Count);

/*
** ReadComm - Reads data from comm port.
**
** hPort: Handle of port to read from.
** pData: Buffer to receive data.
** Count: Number of bytes to read.
**
** Return Value: Number of bytes read.
**
*/
WPLCCOMM_API int ReadComm(HCOMMPORT hPort, void *pData, int Count);

/*
** GetRxAvailable - Returns number of bytes available in the input queue.
**
** hPort: Handle of port to check queue.
**
** Return Value: Number of bytes available.
**
*/
WPLCCOMM_API int GetRxAvailable(HCOMMPORT hPort);

/*
** FlushRxQueue - Clears bytes from receive queue.
**
** hPort: Handle of port to clear.
**
*/
WPLCCOMM_API void FlushRxQueue(HCOMMPORT hPort);

/*
** WaitForInput - Waits for new data in receive queue.
**
** hPort: Handle of port to wait for.
*
** Return Value: Serial event mask
**
*/
WPLCCOMM_API DWORD WaitForInput(HCOMMPORT hPort);

#ifdef __cplusplus
}
#endif

#endif // __WPLCCOMM_H
