#ifndef __WINPLCRT_H
#define __WINPLCRT_H

#ifdef WPLCRT_EXPORTS
#define WPLCRT_API __declspec(dllexport)
#else
#define WPLCRT_API __declspec(dllimport)
#endif

#if !defined(__HEI_H)
#include "hei.h"
#endif

#if defined(__cplusplus)
extern "C"
	{
#endif	/* #if defined(__cplusplus) */

#define FLAG_IN_PARALLEL	0x0001

typedef struct
	{
	BYTE	MajorVersion;
	BYTE	MinorVersion;
	WORD	BuildVersion;
	} WPLCVersionDef;

typedef struct
	{
	WPLCVersionDef OEMVersion;		// Version of OEM Control program.
	WPLCVersionDef WPLCRuntime;		// Version of WPLC runtime code.

	HANDLE hIOFile;

	BYTE Data[128];
	} WPLCInfo;

typedef struct
	{
	DWORD ScanPriority;	// Sets poll thread priority
	DWORD SleepTime;		// -1 = Poll on WPLCPollIO, all others call Sleep(SleepTime);
	} WPLCScanSettings;

WPLCRT_API int WPLCOpen(WPLCInfo *pInfo);
WPLCRT_API int WPLCClose(WPLCInfo *pInfo);
WPLCRT_API int WPLCReadIO(WPLCInfo *pInfo, BYTE *pDataBegin, WORD *pSizeofData);
WPLCRT_API int WPLCWriteIO(WPLCInfo *pInfo, BYTE *pData, WORD DataLen);
WPLCRT_API int WPLCReadBaseDef(WPLCInfo *pInfo, BYTE *pData, WORD *pSize);
WPLCRT_API int WPLCReadModuleStatus(WPLCInfo *pInfo, BYTE *pData, WORD SizeofData);
WPLCRT_API BOOL WPLCLockIOWrites(WPLCInfo *pInfo);
WPLCRT_API BOOL WPLCUnlockIOWrites(WPLCInfo *pInfo);
WPLCRT_API int WPLCConfigIOScan(WPLCInfo *pInfo, WPLCScanSettings *pSS);
WPLCRT_API int WPLCPollIO(WPLCInfo *pInfo);
WPLCRT_API int WPLCGetIOScanCount(WPLCInfo *pInfo, BOOL ResetCount);

WPLCRT_API int WPLCWriteLEDs(WPLCInfo *pInfo, WORD Val);		/* Sets state of Power and Run leds */
WPLCRT_API int WPLCReadSysInfo(WPLCInfo *pInfo, WPLCSystemInformation *pSysInfo);  
WPLCRT_API int WPLCReadVendorCode(WPLCInfo *pInfo, DWORD *pData);

WPLCRT_API int WPLCReadBattRam(DWORD Offset, DWORD Bytes, BYTE *pData);
WPLCRT_API int WPLCWriteBattRam(DWORD Offset, DWORD Bytes, BYTE *pData);
WPLCRT_API int WPLCMapBattRam(DWORD Offset, DWORD Bytes, BYTE **pptr);
WPLCRT_API int WPLCUnMapBattRam(BYTE *ptr, DWORD Bytes);

WPLCRT_API int WPLCReadSetupData(WPLCInfo *pInfo, WORD Type, BYTE *pData, WORD *pSizeofData);
WPLCRT_API int WPLCWriteSetupData(WPLCInfo *pInfo, WORD Type, BYTE *pData, WORD SizeofData);
WPLCRT_API int WPLCEnumSetupData(WPLCInfo *pInfo, WORD *pData, WORD *pSizeofDataInWords);

WPLCRT_API void WPLCWarmReboot(WPLCInfo *pInfo);

WPLCRT_API BOOL WPLCFormatFlashDrive();
WPLCRT_API BOOL WPLCGetFlashDriveFreeSpace(PULARGE_INTEGER lpFreeToCaller, PULARGE_INTEGER lpTotal, PULARGE_INTEGER lpTotalFree);

WPLCRT_API int WPLCSetEPFFun(void (*pFun)());
WPLCRT_API int WPLCGetEPFFun(void (**pFun)());
WPLCRT_API int WPLCReadEPFStatus();

#if defined(__cplusplus)
	}
#endif	/* #if defined(__cplusplus) */

// Errors.
#define WPLC_SIZE_ERROR		0x8200
#define WPLC_INVALID_PARM	0x8201
#define WPLC_WRITES_LOCKED	0x8202

#endif // __WINPLCRT_H

