// IOBase.cpp : Implementation of CIOBase

#include "stdafx.h"
#include <IOBase.h>

CModule::CModule(BYTE Type, BYTE ID, BYTE Xs, BYTE Ys, BYTE WXs, BYTE WYs, BYTE DWXs, BYTE DWYs)
	{
	ModuleType	= Type;
	ModuleID		= ID;
	XCount		= Xs;
	YCount		= Ys;
	WXCount		= WXs;
	WYCount		= WYs;
	DWXCount		= DWXs;
	DWYCount		= DWYs;

//	TCHAR s[100];
//	wsprintf(s, TEXT("XCount = %d; YCount = %d; WXCount = %d; WYCount = %d; DWXCount = %d; DWYCount = %d"), 
//		XCount, YCount, WXCount, WYCount, DWXCount, DWYCount); 
//	OutputDebugString(s);

	memset(X, 0, sizeof(X));
	memset(Y, 0, sizeof(Y));
	memset(WX, 0, sizeof(WX));
	memset(WY, 0, sizeof(WY));
	memset(DWX, 0, sizeof(DWX));
	memset(DWY, 0, sizeof(DWY));
	}

int CModule::WriteModuleState(BYTE *pBuff)
	{
	int Offset = 0;
	if(WYCount)
		{
		pBuff[Offset++] = 0x6;
		pBuff[Offset++] = WYCount * 2;
		for(int idx = 0; idx < WYCount; idx++)
			{
			pBuff[Offset++] = (BYTE)(WY[idx] & 0x00FF);
			pBuff[Offset++] = (BYTE)((WY[idx] & 0xFF00) >> 8);
			}
		}

	if(DWYCount)
		{
		pBuff[Offset++] = 0x9;
		pBuff[Offset++] = DWYCount * 4;
		for(int idx = 0; idx < DWYCount; idx++)
			{
			pBuff[Offset++] = (BYTE)(DWY[idx] & 0x000000FF);
			pBuff[Offset++] = (BYTE)((DWY[idx] & 0x0000FF00) >> 8);
			pBuff[Offset++] = (BYTE)((DWY[idx] & 0x00FF0000) >> 16);
			pBuff[Offset++] = (BYTE)((DWY[idx] & 0xFF000000) >> 24);
			}
		}

	// Move to last to work around WriteIO/HandleType5 problem
	if(YCount)
		{
		int ByteCount = (YCount+7)/8;
		pBuff[Offset++] = 0x4;
		pBuff[Offset++] = ByteCount;
		for(int idx = 0; idx < ByteCount; idx++)
			{
			pBuff[Offset++] = Y[idx];
			}
		}

	return Offset;
	}

void CModule::ClearOutputs()
	{
	memset(Y, 0, sizeof(Y));
	memset(WY, 0, sizeof(WY));
	memset(DWY, 0, sizeof(DWY));
	}

void CIOBase::ParseInputPacket(int Length)
	{
	BOOL Done = FALSE;
	int Slot = 0;
	CModule *pMod = aModules[0];

	WORD Error = (WORD)InBuff[0] | (((WORD)InBuff[1]) << 8);
	
	// Parse input data
	for(int idx = 4; !Done;)
		{
		switch(InBuff[idx])
			{
			case 0x7:
				// Not using base yet
				idx += 2;
				break;

			// Select slot
			case 0x0:
				Slot = InBuff[idx+1];
				pMod = aModules[Slot];
				idx += 2;
				break;

			// Discrete in
			case 0x3:
				if(pMod)
					pMod->SetXState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Discrete out
			case 0x4:
				if(pMod)
					pMod->SetYState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Word in
			case 0x5:
				if(pMod)
					pMod->SetWXState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Word out
			case 0x6:
				if(pMod)
					pMod->SetWYState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Dword in
			case 0x8:
				if(pMod)
					pMod->SetDWXState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Dword out
			case 0x9:
				if(pMod)
					pMod->SetDWYState(InBuff+idx+2, InBuff[idx+1]);

				idx += 2 + InBuff[idx+1];
				break;

			// Complete
			case 0xFF:
				Done = TRUE;
				break;

			default:
				idx += 2 + InBuff[idx+1];
				break;
			}
		}
	}

int CIOBase::PackOutputPacket()
	{
	int Offset = 0;
	OutBuff[Offset++] = 0xB0;
	OutBuff[Offset++] = 0x00;
	for(int idx = 0; idx < 8; idx++)
		{
		if(aModules[idx] && aModules[idx]->IsOut())
			{
			OutBuff[Offset++] = 0;
			OutBuff[Offset++] = idx;
			Offset += aModules[idx]->WriteModuleState(OutBuff+Offset);
			}
		}

	OutBuff[Offset++] = 0xFF;
	return Offset;
	}


/////////////////////////////////////////////////////////////////////////////
// CIOBase
CIOBase::CIOBase()
{
	memset(&Info, 0, sizeof(Info));
	memset(aModules, 0, sizeof(aModules));
}

int CIOBase::Init()
{
	/*
	** Open winplc runtime
	*/
	WPLCOpen(&Info);

	/*
	** Read base contents
	*/
	BYTE BaseDef[200];
	WORD BaseDefSize = sizeof(BaseDef);
	int RetVal = WPLCReadBaseDef(&Info, BaseDef, &BaseDefSize);

	/*
	** Parse response and create modules
	*/
	BOOL Done = FALSE;
	int Slot = -1;
	int Base = 0;

	// Create module defs
	for(int idx = 0; idx < BaseDefSize && !Done;)
		{
		switch(BaseDef[idx])
			{
			case 0x7:
				Base = BaseDef[idx+1];
				idx += 2;
				break;

			case 0x0:
				Slot = BaseDef[idx+1];
				idx += 2;
				break;

			case 0x1:
				if(!aModules[Slot])
					{
					if(BaseDef[idx+2] == 1)
						{
						aModules[Slot] = new CModule(BaseDef[idx+2], BaseDef[idx+3], BaseDef[idx+4], 
							BaseDef[idx+5], BaseDef[idx+6], BaseDef[idx+7], 0, 0);
						}
					else if(BaseDef[idx+2] == 5)
						{
						aModules[Slot] = new CModule(BaseDef[idx+2], BaseDef[idx+3], BaseDef[idx+4], 
							BaseDef[idx+5], BaseDef[idx+6], BaseDef[idx+7], BaseDef[idx+8], BaseDef[idx+9]);
						}
					}

				idx += 4 + BaseDef[idx+1];
				break;

			case 0xFF:
				Done = TRUE;
				idx++;
				break;

			default:
				idx += 2 + BaseDef[idx+1];
				break;
			}
		}

	/*
	** Calc IO offsets
	*/
	WORD CurX = 0, CurY = 0, CurWX = 0, CurWY = 0, CurDWX = 0, CurDWY = 0;
	for(idx = 0; idx < 8; idx++)
		{
		CModule *pMod = aModules[idx];
		if(pMod)
			{
			if(pMod->XCount)
				{
				CurX += pMod->XCount;
				aXInSlot[idx] = CurX;
				}

			if(pMod->YCount)
				{
				CurY += pMod->YCount;
				aYInSlot[idx] = CurY;
				}

			if(pMod->WXCount)
				{
				CurWX += pMod->WXCount;
				aWXInSlot[idx] = CurWX;
				}

			if(pMod->WYCount)
				{
				CurWY += pMod->WYCount;
				aWYInSlot[idx] = CurWY;
				}

			if(pMod->DWXCount)
				{
				CurDWX += pMod->DWXCount;
				aDWXInSlot[idx] = CurDWX;
				}

			if(pMod->DWYCount)
				{
				CurDWY += pMod->DWYCount;
				aDWYInSlot[idx] = CurDWY;
				}
			}
		else
			{
			aXInSlot[idx] = 0;
			aYInSlot[idx] = 0;
			aWXInSlot[idx] = 0;
			aWYInSlot[idx] = 0;
			aDWXInSlot[idx] = 0;
			aDWYInSlot[idx] = 0;
			}
		}

	/*
	** Read IO
	*/
	ReadIO();

	return TRUE;
}

int CIOBase::Exit()
{
	// Shut down the base interface
	WPLCClose(&Info);

	// Clean up
	for(int idx = 0; idx < 8; idx++)
		{
		delete aModules[idx];
		aModules[idx] = NULL;
		}
	return TRUE;
}

int CIOBase::OnIdle(int SleepTime, int PollCount, BOOL ClearScanCount)
{ 
	Sleep(SleepTime);
	for(int idx = 0; idx < PollCount; idx++) PollIO();

	return GetIOScanCount(ClearScanCount);
}

BOOL CIOBase::LockIOWrites()
{
	return WPLCLockIOWrites(&Info);
}

BOOL CIOBase::UnlockIOWrites()
{
	return WPLCUnlockIOWrites(&Info);
}

int CIOBase::ConfigIOScan(DWORD ScanPriority, DWORD SleepTime)
{
	WPLCScanSettings SS;
	SS.ScanPriority = ScanPriority;
	SS.SleepTime = SleepTime;
	return WPLCConfigIOScan(&Info, &SS);
}

int CIOBase::PollIO()
{
	return WPLCPollIO(&Info);
}

int CIOBase::GetIOScanCount(BOOL ResetCount)
{
	return WPLCGetIOScanCount(&Info, ResetCount);
}

int CIOBase::ReadIO()
{
	WORD Length = 0;

	WPLCReadIO(&Info, InBuff, &Length);

	ParseInputPacket(Length);
	return TRUE;
}

int CIOBase::WriteIO()
{
	WORD Length = PackOutputPacket();
	WPLCWriteIO(&Info, OutBuff, Length);
	return TRUE;
}

void CIOBase::OutputDebugString(LPCSTR pStr)
{
	OutputDebugString(pStr);
	Sleep(10);
}

#ifdef DEBUG_IO
BOOL CIOBase::GetDI(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->XCount)
				{
				return aModules[Slot]->GetX(Point);
				}
			}
		}

	return -1;
}

BOOL CIOBase::GetDO(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->YCount)
				{
				return aModules[Slot]->GetY(Point);
				}
			}
		}

	return -1;
}

void CIOBase::SetDO(short Slot, short Point, BOOL NewVal)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->YCount)
				{
				aModules[Slot]->SetY(Point, NewVal);
				}
			}
		}
}

WORD CIOBase::GetWI(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WXCount)
				{
				return aModules[Slot]->WX[Point];
				}
			}
		}

	return -1;
}

WORD CIOBase::GetWO(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WYCount)
				{
				return aModules[Slot]->WY[Point];
				}
			}
		}

	return -1;
}

void CIOBase::SetWO(short Slot, short Point, WORD NewVal)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WYCount)
				{
				aModules[Slot]->WY[Point] = NewVal;
				}
			}
		}
}

DWORD CIOBase::GetDWI(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWXCount)
				{
				return aModules[Slot]->DWX[Point];
				}
			}
		}

	return -1;
}

DWORD CIOBase::GetDWO(short Slot, short Point)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWYCount)
				{
				return aModules[Slot]->DWY[Point];
				}
			}
		}

	return -1;
}

void CIOBase::SetDWO(short Slot, short Point, DWORD NewVal)
{
	if(Slot < 8)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWYCount)
				{
				aModules[Slot]->DWY[Point] = NewVal;
				}
			}
		}
}
#endif

void CIOBase::ClearOutputs()
{
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aModules[Slot])
			aModules[Slot]->ClearOutputs();
		}

}

void CIOBase::TurnOffRunLED()
{
	WPLCWriteLEDs(&Info, WPLC_LED_POWER);
}

void CIOBase::TurnOnRunLED()
{
	WPLCWriteLEDs(&Info, WPLC_LED_POWER | WPLC_LED_RUN);
}

BOOL CIOBase::GetX(short Point)
{
	WORD LastX = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aXInSlot[Slot])
			{
			if(Point >= LastX && Point < aXInSlot[Slot])
				return aModules[Slot]->GetX(Point - LastX);

			LastX += aXInSlot[Slot];
			}
		}

	return -1;
}

BOOL CIOBase::GetY(short Point)
{
	WORD LastY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aYInSlot[Slot])
			{
			if(Point >= LastY && Point < aYInSlot[Slot])
				return aModules[Slot]->GetY(Point - LastY);

			LastY += aYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CIOBase::SetY(short Point, BOOL NewVal)
{
	WORD LastY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aYInSlot[Slot])
			{
			if(Point >= LastY && Point < aYInSlot[Slot])
				{
				aModules[Slot]->SetY(Point - LastY, NewVal);
				return 0;
				}
	
			LastY += aYInSlot[Slot];
			}
		}

	return -1;
}

WORD CIOBase::GetWX(short Point)
{
	WORD LastWX = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aWXInSlot[Slot])
			{
			if(Point >= LastWX && Point < aWXInSlot[Slot])
				return aModules[Slot]->WX[Point-LastWX];

			LastWX += aWXInSlot[Slot];
			}
		}

	return -1;
}

WORD CIOBase::GetWY(short Point)
{
	WORD LastWY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aWYInSlot[Slot])
			{
			if(Point >= LastWY && Point < aWYInSlot[Slot])
				return aModules[Slot]->WY[Point-LastWY];

			LastWY += aWYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CIOBase::SetWY(short Point, WORD NewVal)
{
	WORD LastWY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aWYInSlot[Slot])
			{
			if(Point >= LastWY && Point < aWYInSlot[Slot])
				{
				aModules[Slot]->WY[Point-LastWY] = NewVal;
				return 0;
				}

			LastWY += aWYInSlot[Slot];
			}
		}

	return -1;
}

DWORD CIOBase::GetDWX(short Point)
{
	WORD LastDWX = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aDWXInSlot[Slot])
			{
			if(Point >= LastDWX && Point < aDWXInSlot[Slot])
				return aModules[Slot]->DWX[Point-LastDWX];

			LastDWX += aDWXInSlot[Slot];
			}
		}

	return -1;
}

DWORD CIOBase::GetDWY(short Point)
{
	WORD LastDWY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aDWYInSlot[Slot])
			{
			if(Point >= LastDWY && Point < aDWYInSlot[Slot])
				return aModules[Slot]->DWY[Point-LastDWY];

			LastDWY += aDWYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CIOBase::SetDWY(short Point, DWORD NewVal)
{
	WORD LastDWY = 0;
	for(int Slot = 0; Slot < 8; Slot++)
		{
		if(aDWYInSlot[Slot])
			{
			if(Point >= LastDWY && Point < aDWYInSlot[Slot])
				{
				aModules[Slot]->DWY[Point-LastDWY] = NewVal;
				return 0;
				}

			LastDWY += aDWYInSlot[Slot];
			}
		}

	return -1;
}

